"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var _workspaceResolver = require("@modular-scripts/workspace-resolver");

var _actionPreflightCheck = _interopRequireDefault(require("../utils/actionPreflightCheck"));

var _resolve = _interopRequireDefault(require("resolve"));

var _execAsync = _interopRequireDefault(require("../utils/execAsync"));

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _getAllWorkspaces = require("../utils/getAllWorkspaces");

var _getChangedWorkspaces = require("../utils/getChangedWorkspaces");

var _resolveAsBin = require("../utils/resolveAsBin");

var logger = _interopRequireWildcard(require("../utils/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// via https://github.com/facebook/create-react-app/blob/master/packages/react-scripts/scripts/test.js
// This is a very dirty workaround for https://github.com/facebook/jest/issues/5913.
// We're trying to resolve the environment ourselves because Jest does it incorrectly.
// TODO: remove this as soon as it's fixed in Jest.
function resolveJestDefaultEnvironment(name) {
  const jestDir = path.dirname(_resolve.default.sync('jest', {
    basedir: __dirname
  }));
  const jestCLIDir = path.dirname(_resolve.default.sync('jest-cli', {
    basedir: jestDir
  }));
  const jestConfigDir = path.dirname(_resolve.default.sync('jest-config', {
    basedir: jestCLIDir
  }));
  return _resolve.default.sync(name, {
    basedir: jestConfigDir
  });
}

async function test(options, userRegexes) {
  const {
    ancestors,
    changed,
    package: packages,
    compareBranch,
    debug,
    env,
    reporters,
    testResultsProcessor,
    ...jestOptions
  } = options; // create argv from jest Options

  const cleanArgv = []; // pass in path to configuration file

  const {
    createJestConfig
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('./config')));
  cleanArgv.push('--config', `"${JSON.stringify(createJestConfig({
    reporters,
    testResultsProcessor
  }))}"`);
  let resolvedEnv;

  try {
    resolvedEnv = resolveJestDefaultEnvironment(`jest-environment-${env}`);
  } catch (e) {// ignore
  }

  if (!resolvedEnv) {
    try {
      resolvedEnv = resolveJestDefaultEnvironment(env);
    } catch (e) {// ignore
    }
  }

  const testEnvironment = resolvedEnv || env;
  cleanArgv.push(`--env=${testEnvironment}`); // pass on all programatic options

  const jestArgv = Object.entries(jestOptions).map(([key, v]) => {
    const booleanValue = /^(true)$/.exec(String(v));
    return `--${key}${!!booleanValue ? '' : `=${String(v)}`}`;
  });
  cleanArgv.push(...jestArgv);
  const additionalOptions = [];
  const cleanRegexes = []; // There are two ways of discovering the test regexes we need: either they're specified by the user as CLI arguments
  // or they have to be calculated from selective options (--changed and --package) and optionally agumented with --ancestors

  const isSelective = changed || (packages === null || packages === void 0 ? void 0 : packages.length);
  const regexes = isSelective ? await computeSelectiveRegexes({
    changed,
    compareBranch,
    packages,
    ancestors
  }) : userRegexes; // If test is selective (user set --changed or --package) and we computed no regexes, then bail out

  if (!(regexes !== null && regexes !== void 0 && regexes.length) && isSelective) {
    process.stdout.write(changed ? 'No changed workspaces found\n' : 'No workspaces found in selection\n');
    process.exit(0);
  }

  if (regexes !== null && regexes !== void 0 && regexes.length) {
    regexes.forEach(reg => {
      if (/^(--)([\w]+)/.exec(reg)) {
        return additionalOptions.push(reg);
      }

      return cleanRegexes.push(reg);
    });

    if (additionalOptions.length) {
      additionalOptions.map(reg => {
        const [option, value] = reg.split('=');

        if (value) {
          return `${option}=${JSON.stringify(value)}`;
        }

        return option;
      });
    }
  } // push any additional options passed in by debugger or other processes


  cleanArgv.push(...additionalOptions); // finally add the script regexes to run

  cleanArgv.push(...cleanRegexes);
  const jestBin = await (0, _resolveAsBin.resolveAsBin)('jest-cli');
  let testBin = jestBin,
      testArgs = cleanArgv;

  if (debug) {
    // If we're trying to attach to a debugger, we need to run node
    // instead. This moves around the command line arguments for so.
    testBin = 'node';
    testArgs = ['--inspect-brk', jestBin, ...testArgs.filter(x => x !== '--inspect-brk')];
  }

  try {
    await (0, _execAsync.default)(testBin, testArgs, {
      cwd: (0, _getModularRoot.default)(),
      log: false,
      // @ts-ignore
      env: {
        BABEL_ENV: 'test',
        NODE_ENV: 'test',
        PUBLIC_URL: '',
        MODULAR_ROOT: (0, _getModularRoot.default)()
      }
    });
  } catch (err) {
    logger.debug(err.message); // ✕ Modular test did not pass

    throw new Error('\u2715 Modular test did not pass');
  }
} // This function takes all the selective options, validates them and returns a subset of workspaces to test


async function computeSelectiveRegexes({
  changed,
  compareBranch,
  packages,
  ancestors
}) {
  logger.debug(packages !== null && packages !== void 0 && packages.length ? `Calculating test regexes from specified packages: ${JSON.stringify(packages)}` : `Calculating test regexes from changed workspaces, compared with ${compareBranch ?? 'default'} branch`);

  if (!changed && !(packages !== null && packages !== void 0 && packages.length) || changed && packages !== null && packages !== void 0 && packages.length) {
    throw new Error(`Conflicting options: --changed (${changed.toString()}) and --package ("${JSON.stringify(packages)}")`);
  } // It's terser if we mutate WorkspaceContent; we will use it only to pass it to computeTestsRegexes


  let resultWorkspaceContent = packages !== null && packages !== void 0 && packages.length ? await getSinglePackagesContent(packages) : await (0, _getChangedWorkspaces.getChangedWorkspacesContent)(compareBranch);

  if (ancestors) {
    logger.debug(`Calculating ancestors of packages: ${JSON.stringify(Object.keys(resultWorkspaceContent[1]))}`);
    resultWorkspaceContent = await getAncestorWorkspacesContent(resultWorkspaceContent);
  }

  logger.debug(`Selected test packages are: ${JSON.stringify(Object.keys(resultWorkspaceContent[1]))}`);
  return computeTestsRegexes(resultWorkspaceContent);
} // This function returns a WorkspaceContent from an array of workspace names


async function getSinglePackagesContent(singlePackages) {
  // Get all the workspaces
  const allWorkspaces = await (0, _getAllWorkspaces.getAllWorkspaces)((0, _getModularRoot.default)());
  const uniqueSinglePackages = Array.from(new Set(singlePackages));
  const result = [new Map(), {}];
  const [sourcePackageContent, sourcePackageMap] = allWorkspaces;
  const [targetPackageContent, targetPackageMap] = result; // Filter, copy onto result and validate existence. This is easier to express with a mutable for loop

  for (const pkgName of uniqueSinglePackages) {
    const packageContent = sourcePackageContent.get(pkgName);

    if (!sourcePackageMap[pkgName] || !packageContent) {
      throw new Error(`Package ${pkgName} was specified, but Modular couldn't find it`);
    }

    targetPackageContent.set(pkgName, packageContent);
    targetPackageMap[pkgName] = sourcePackageMap[pkgName];
  }

  return result;
} // This function takes a WorkspaceContent and returns a WorkspaceContent agumented with all the ancestors of the original one


async function getAncestorWorkspacesContent(selectedWorkspaces) {
  const allWorkspaces = await (0, _getAllWorkspaces.getAllWorkspaces)((0, _getModularRoot.default)());
  return (0, _workspaceResolver.computeAncestorWorkspaces)(selectedWorkspaces, allWorkspaces);
} // This function returns a list for test regexes from a WorkspaceContent


function computeTestsRegexes(selectedWorkspaces) {
  const testRegexes = Object.values(selectedWorkspaces[1]).map(({
    location
  }) => location);
  return testRegexes;
}

var _default = (0, _actionPreflightCheck.default)(test);

exports.default = _default;
//# sourceMappingURL=index.js.map