"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectWorkspaces = selectWorkspaces;

var _workspaceResolver = require("@modular-scripts/workspace-resolver");

var _getAllWorkspaces = require("./getAllWorkspaces");

var _getChangedWorkspaces = require("./getChangedWorkspaces");

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function selectWorkspaces({
  targets,
  changed,
  ancestors,
  descendants,
  compareBranch
}) {
  const [, allWorkspacesMap] = await (0, _getAllWorkspaces.getAllWorkspaces)((0, _getModularRoot.default)());
  let changedTargets = [];

  if (changed) {
    const [, buildTargetMap] = await (0, _getChangedWorkspaces.getChangedWorkspacesContent)(compareBranch);
    changedTargets = Object.keys(buildTargetMap);
  }

  const targetsToBuild = [...new Set(targets.concat(changedTargets))];

  if (!targetsToBuild.length) {
    return [];
  } // Calculate the package scope, i.e. the deduped array of all the packages we need to generate an ordering graph for.
  // We need to remember ancestor and descendant sets in order to select them later.


  let ancestorsSet = new Set();
  let descendantsSet = new Set();
  let packageScope = targetsToBuild;

  if (descendants) {
    descendantsSet = (0, _workspaceResolver.computeDescendantSet)(targetsToBuild, allWorkspacesMap);
    packageScope = packageScope.concat([...descendantsSet]);
  }

  if (ancestors) {
    ancestorsSet = (0, _workspaceResolver.computeAncestorSet)(targetsToBuild, allWorkspacesMap);
    packageScope = packageScope.concat([...ancestorsSet]);
  }

  packageScope = [...new Set(packageScope)]; // traverseWorkspaceRelations will walk the graph and generate the whole ordering needed to build a package
  // which means that it will possibly expand the scope (it just generates the dependency order starting from a dependency subset but taking into account the whole dependency graph)
  // this means that we can build in order even manually selected subsets of packages (like: "the user wants to select only a and b, but execute tasks on them in order"),
  // but it also means we need to filter out all the unwanted packages later.

  const targetEntriesWithOrder = [...(0, _workspaceResolver.traverseWorkspaceRelations)(packageScope, allWorkspacesMap).entries()];
  return targetEntriesWithOrder.sort((a, b) => b[1] - a[1]).map(([packageName]) => packageName) // Filter out descendants and ancestors if we don't explicitly need them.
  // This allows us to get the correct dependency order even if we restrict the scope (for example, by explicit user input).
  .filter(packageName => descendants && descendantsSet.has(packageName) || ancestors && ancestorsSet.has(packageName) || targetsToBuild.includes(packageName));
}
//# sourceMappingURL=selectWorkspaces.js.map